//*************************************************************************
//* --------------------
//* class S4BreakPoint
//* --------------------
//* (Description)
//*     Class to book keep break ponit information.
//* (Update Record)
//*     2006/01/29  S.Yamamoto     Original version.
//*************************************************************************

#include "S4BreakPoint.h"
#include "S4HISTMCTruthBuf.h"
#include "TString.h"
#include <iostream>
#include <iomanip>

std::map<Int_t, S4BreakPoint*> S4BreakPoint::fgBreakPointMap; 

ClassImp(S4BreakPoint)

//=====================================================================
//* Constructors ------------------------------------------------------
S4BreakPoint::S4BreakPoint()
            : fTrackID(0),
	      fParentID(0),
	      fCharge(0),
	      fParticleID(0),
	      fGlobalTime(0),
	      fKineticEnergy(0),
	      fPFOPtr(0),
	      fPHitPtr(0)
{
}

S4BreakPoint::S4BreakPoint(      Int_t     hitID,       // hit ID
                                 Int_t     trackID,     // track ID
                                 Int_t     mtrackID,    // mother track ID
                                 Int_t     pcode,       // PDG code
                                 Double_t  charge,      // charge
                           const TVector3 &pos,         // position [cm]
                                 Double_t  tof,         // TOF      [nsec]
                           const TVector3 &mom,         // momentum [GeV]
	                         Double_t  energy)      // energy [GeV]
            : fTrackID(trackID),
	      fParentID(mtrackID),
	      fCharge(charge),
	      fParticleID(pcode),
	      fGlobalTime(tof),
	      fPosition(pos),
	      fFourMomentum(mom,energy),
	      fKineticEnergy(energy-fFourMomentum.M()),
	      fPFOPtr(0),
	      fPHitPtr(0)
{
}

//=====================================================================
//* Destructor -------------------------------------------------------
S4BreakPoint::~S4BreakPoint()
{
}

//=====================================================================
//* GetBreakPointPtr --------------------------------------------------
S4BreakPoint * S4BreakPoint::GetBreakPointPtr(Int_t trackID)
{
  std::map<Int_t, S4BreakPoint *>::iterator iter = fgBreakPointMap.find(trackID);
  if (iter != fgBreakPointMap.end()) {
    return iter->second;
  } else {
    return 0;
  }
}

//=====================================================================
//* Clear -------------------------------------------------------------
void S4BreakPoint::Clear()
{
  fgBreakPointMap.clear();
}

//=====================================================================
//* GetBreakPointMap --------------------------------------------------
std::map<Int_t, S4BreakPoint *> & S4BreakPoint::GetBreakPointMap()
{
  return fgBreakPointMap;
}

//=====================================================================
//* LockAllDescendants ------------------------------------------------
void S4BreakPoint::LockAllDescendants()
{

  TIter nextchild(this);
  S4BreakPoint *cbpp;
  while ((cbpp = static_cast<S4BreakPoint *>(nextchild()))) {
    cbpp->Lock();
    TAttLockable *cpfop = dynamic_cast<TAttLockable *>(cbpp->GetPFOPtr());
    if (cpfop)  cpfop->Lock();
    cbpp->LockAllDescendants();
  }   
}

//=====================================================================
//* DebugPrint --------------------------------------------------------

void S4BreakPoint::DebugPrint(const Char_t *opt)
{
  using namespace std;
  TString sopt(opt);
  sopt.ToLower();
  if (sopt.Contains("head")) {
    cerr << "+----+---------+---------+---------+"
         <<       "---------+---------+---------+---------+"
         <<       "---------+"
         <<       "---------+---------+---------+---------+"
         <<       "---------+"
         <<       "---------+"
         << endl
         << " lock   trackID  mtrackID     PID   "
         <<     "    E         Px        Py       Pz     "
         <<     "    Ekin  "
         <<     "    T         Vx        Vy       Vz     "
         <<     "    Rho   "
         <<     "  pfoPtr  "
         << endl;
    cerr << "+----+---------+---------+---------+"
         <<       "---------+---------+---------+---------+"
         <<       "---------+"
         <<       "---------+---------+---------+---------+"
         <<       "---------+"
         <<       "---------+"
         << endl;
    return;
  }
  cerr << setiosflags(ios::fixed);
  cerr << setw( 5) << (IsLocked() ? "Y" : "N") << " "
       << setw( 9) << GetTrackID()             << " "
       << setw( 9) << GetParentID()            << " "	
       << setw( 9) << GetParticleID()          << " "
       << setprecision(4)
       << setw( 9) << GetFourMomentum().T()    << " "
       << setw( 9) << GetFourMomentum().X()    << " "
       << setw( 9) << GetFourMomentum().Y()    << " "
       << setw( 9) << GetFourMomentum().Z()    << " "
       << setw( 9) << GetKineticEnergy()       << " "
       << setw( 9) << GetGlobalTime()          << " "
       << setw( 9) << GetPosition().X()        << " "
       << setw( 9) << GetPosition().Y()        << " "
       << setw( 9) << GetPosition().Z()        << " "
       << setw( 9) << GetPosition().Perp()     << " "
       << setw( 9) << GetPFOPtr()              << " "
       << endl;
  cerr << resetiosflags(ios::fixed);
  if (sopt.Contains("rec")) {
    TIter next(this);
    S4BreakPoint *bpp;
    while ((bpp = static_cast<S4BreakPoint *>(next()))) {
      if (bpp) bpp->DebugPrint(opt);
    }
  }
}

//=====================================================================
//* PrintAll ----------------------------------------------------------
void S4BreakPoint::PrintAll(const Char_t *opt)
{
  using namespace std;
  TString opts(opt);
  opts.ToLower();
  cerr << "--------------------------------------------- " << endl
       << "S4BreakPoint::PrintAll() -------------------- " << endl;
  Double_t ecmbp = 0;
  std::map<Int_t, S4BreakPoint*>::iterator iter;
  for (iter  = fgBreakPointMap.begin(); 
       iter != fgBreakPointMap.end(); iter++) {
    S4BreakPoint *bpp = iter->second;
    if (iter == fgBreakPointMap.begin()) bpp->DebugPrint("head");
    if (opts.Contains("primaryonly")) {
      if (bpp->TestBit(BIT(20))) bpp->DebugPrint(opt);
    } else if (opts.Contains("withpfo")) {
      if (bpp->GetPFOPtr()) bpp->DebugPrint(opt);
    } else {
      bpp->DebugPrint(opt);
    }
    if (bpp->TestBit(BIT(20))) ecmbp += bpp->GetFourMomentum().E();
    if (*iter == *fgBreakPointMap.rbegin()) bpp->DebugPrint("head");
  }
  cerr << "--------------------------------------------- " << endl
       << "   Ecm (BP) = " << ecmbp << " [GeV]"            << endl
       << "--------------------------------------------- " << endl;
}
