//*****************************************************************************
//* -----------------
//* class S4VExactHit
//* -----------------
//*
//* (Purpose)
//*   Base class for exact hit objects to provide basic functions to access
//*   exact hit information from Jupiter.
//*
//* (Dependency)
//*   Inherits from:   TObject, TAttLockable
//*   Uses:            TVector3, TVAddress, S4VHit
//* (Usage)
//*
//* (Update Record)
//*   2004/07/30  K.Fujii  Primitive version. 
//*
//*****************************************************************************

#include "S4VExactHit.h"
#include <iostream>
#if __GNUC__ >= 5 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3)
#include <cstdlib>
#endif

ClassImp(S4VExactHit)

S4VExactHit::ESortOption S4VExactHit::fgSortBy = kTrackID;

//=============================================================================
//* constructor ---------------------------------------------------------------

S4VExactHit::S4VExactHit()
  : fHitID(-9999), 
    fTrackID(-9999),
    fMotherTrackID(-9999),
    fPDGCode(-99999999),
    fCharge(0.),
    fPosition(0.,0.,0.), 
    fTOF(0.),
    fMomentum(0.,0.,0.),
    fEnergy(0.),
    fEnergyDeposit(0.),
    fAddressPtr(0),
    fSmearedHitPtr(0),
    fEventID(-99999)
{
}

S4VExactHit::S4VExactHit(Int_t       hitid,      // hist ID
                         Int_t       trackid,    // track ID
                         Int_t       momtrackid, // mother track ID
                         Int_t       pid,        // PDG code
                         Double_t    chg,        // charge
                   const TVector3  & xv,         // position
                         Double_t    tof,        // TOF
                   const TVector3  & pv,         // momentum
                         Double_t    e,          // energy
                         Double_t    edep,       // energy deposit
                         TVAddress * addr,       // address
                         Int_t       evid,       // event ID
                         void      * shtp)       // backward pointer to smeared hit
  : fHitID(hitid), 
    fTrackID(trackid),
    fMotherTrackID(momtrackid),
    fPDGCode(pid),
    fCharge(chg),
    fPosition(xv), 
    fTOF(tof),
    fMomentum(pv),
    fEnergy(e),
    fEnergyDeposit(edep),
    fAddressPtr(addr),
    fSmearedHitPtr(shtp),
    fEventID(evid)
{
}


//=============================================================================
//* Utility functions ---------------------------------------------------------

Bool_t S4VExactHit::IsSortable() const
{
   return kTRUE;
}

Int_t  S4VExactHit::Compare(const TObject *obj) const
{
   switch(GetSortBy()) {
      case kHitID: {
         Int_t    hid = dynamic_cast<const S4VExactHit *>(obj)->GetHitID();
         return (fHitID  == hid ? 0 : (fHitID < hid ? -1 : 1));
         break;
      }
      case kTrackID: {
         Int_t    tid = dynamic_cast<const S4VExactHit *>(obj)->GetTrackID();
         return (fTrackID  == tid ? 0 : (fTrackID < tid ? -1 : 1));
         break;
      }
      case kMotherTrackID: {
         Int_t   mtid = dynamic_cast<const S4VExactHit *>(obj)->GetMotherTrackID();
         return (fMotherTrackID  == mtid ? 0 : (fMotherTrackID < mtid ? -1 : 1));
         break;
      }
      case kPDGCode: {
         Int_t    pdg = dynamic_cast<const S4VExactHit *>(obj)->GetPDGCode();
         return (fPDGCode  == pdg ? 0 : (fPDGCode < pdg ? -1 : 1));
         break;
      }
      case kCharge: {
         Double_t chg = dynamic_cast<const S4VExactHit *>(obj)->GetCharge();
         return (fCharge  == chg ? 0 : (fCharge < chg ? -1 : 1));
         break;
      }
      case kEnergy: {
         Double_t e = dynamic_cast<const S4VExactHit *>(obj)->GetTotalEnergy();
         return (fEnergy  == e ? 0 : (fEnergy < e ? -1 : 1));
         break;
      }
      case kEnergyDeposit: {
         Double_t ed = dynamic_cast<const S4VExactHit *>(obj)->GetEnergyDeposit();
         return (fEnergyDeposit  == ed ? 0 : (fEnergyDeposit < ed ? -1 : 1));
         break;
      }
      case kTof: {
         Double_t t = dynamic_cast<const S4VExactHit *>(obj)->GetTOF();
         return (fTOF  == t ? 0 : (fTOF < t ? -1 : 1));
         break;
      }
      case kEventID: {
         Int_t    eid = dynamic_cast<const S4VExactHit *>(obj)->GetEventID();
         return (fEventID  == eid ? 0 : (fEventID < eid ? -1 : 1));
         break;
      }
      case kPosition:
      case kMomentum:
      case kAddress:
      default:
         std::cerr << ">>>> S4VExactHit::Compare >>>>" << std::endl
                   << " Sort option = " << GetSortBy()
                   << " is currently not supported."    
                   << " Abort!! "                      << std::endl;
         abort();
         break;
   }
}
