#include "S4CALCluster.h"
#include "S4CALHit.h"
#include "S4CALHitFilter.h"
#include "S4CALExactHit.h"

#include "TVirtualPad.h"
#include "TPolyMarker3D.h"
#include "TString.h"
#include <iostream>
#include <limits>

S4CALHitFilter *S4CALCluster::fgFilterPtr   = 0;

Double_t S4CALCluster::fgEMCut  = 0.9;          // cut for CalibrationFactor
Double_t S4CALCluster::fgCalibFactorForEM = 37.76588/1.055; // calibfactor for EM
Double_t S4CALCluster::fgCalibFactorForHD = 33.37009/1.0816; // calibfactor for HD

ClassImp(S4CALCluster)

S4CALCluster::S4CALCluster(S4CALHit *chp)
            : fEnergy (0.),
              fTime   (0.),
              fFracEM (0.),
              fDone   (kFALSE)
{
  if (chp) Append(chp);
}

S4CALCluster::~S4CALCluster()
{
}

Bool_t S4CALCluster::Append(S4CALHit *chp)
{
  fDone = kFALSE;
  // cheated clustering
  if (!GetEntries() || 
      static_cast<S4CALHit *>(Last())->GetPreHitID() == chp->GetPreHitID()) {
    Add(chp);
    return kTRUE;
  } else {
    return kFALSE;
  }
}

S4CALHitFilter &S4CALCluster::GetFilter()
{
  if (!fgFilterPtr) fgFilterPtr   = new S4CALHitFilter();
  return *fgFilterPtr;
}

void S4CALCluster::SetFilter(S4CALHitFilter *fp)
{
  delete fgFilterPtr;
  fgFilterPtr = fp;
}

void S4CALCluster::Update()
{
  if (fDone) return;
  fDone = kTRUE;

  fEnergy = 0.;
  fTime   = std::numeric_limits<int>::max();
  fPosition.SetXYZ(0.,0.,0.);

  Double_t eem = 0.;
  TIter next(this);
  S4CALHit *chp;
  while ((chp = static_cast<S4CALHit *>(next()))) {
    if (GetFilter().IsOK(*chp)) {
       Double_t edep = chp->GetEnergyDeposit();
       fEnergy   += edep;
       fPosition += edep * chp->GetPosition();
       if (chp->GetTime() < fTime) fTime = chp->GetTime();
       if (chp->GetCALType()&S4CALHit::kBaEm || 
           chp->GetCALType()&S4CALHit::kEcEm) eem += edep;
    }
  }
  fPosition *= 1./fEnergy;
  fFracEM    = eem/fEnergy;

  fEnergy   *= fFracEM < fgEMCut ? fgCalibFactorForHD : fgCalibFactorForEM;
  fMomentum  = fEnergy * fPosition.Unit(); 
}

void S4CALCluster::Draw(Int_t color, const Char_t *opt)
{
  if (!gPad || !GetEntries()) return;
  gPad->cd();
#if 0
  TIter next(this);
  S4CALHit *chp;
  while ((chp = static_cast<S4CALHit *>(next()))) {
    chp->Draw(color, opt);
  }
#else
  TPolyMarker3D *pm3dp = new TPolyMarker3D(GetEntries());
  pm3dp->SetBit(kCanDelete);
  pm3dp->SetMarkerColor(color);

  Int_t nhits = 0;
  TIter next(this);
  S4CALHit *chp = 0;
  while ((chp = static_cast<S4CALHit *>(next()))) {
    if (chp->IsLocked()) continue;
    TVector3 xv   = chp->GetPosition();
    // Double_t edep = chp->GetEnergyDeposit();
    pm3dp->SetPoint(nhits, xv.X(), xv.Y(), xv.Z());
    nhits++;
  }
  pm3dp->Draw();
#endif
  gPad->Update();
}

void S4CALCluster::DebugPrint(const Char_t *opt)
{
  S4CALHit      *chp = static_cast<S4CALHit *>(At(0));
  S4CALExactHit *ehp = static_cast<S4CALExactHit *>(chp->At(0));
  using namespace std;
  cerr << " -- S4CALCluster::DebugPrint -------------------- " << endl
       << "  phtid = " << chp->GetPreHitID()                   << endl
       << "  clPtr = " << this                                 << endl
       << "  Nhits = " << GetEntries()                         << endl
       << "  Theta = " << GetPosition().Theta()                << endl
       << "  Phi   = " << GetPosition().Phi()                  << endl
       << "  R     = " << GetPosition().Mag()                  << endl
       << "  E     = " << GetEnergy()                          << endl
       << "  pdgc  = " << ehp->GetPDGCode()                    << endl
       << " ------------------------------------------------ " << endl;
  TString opts(opt);
  opts.ToLower();
  if (opts.Contains("very detail")) {
    TIter next(this);
    S4CALHit *chp;
    while ((chp = static_cast<S4CALHit *>(next()))) {
      chp->DebugPrint(opt);
    }
  }
}
